#!/bin/bash

# Install PHP with SQLite+IMAP (if not already included)

# If we haven't read functions.sh by now, then read it
if [ "x$ZTFUNCTIONS" = "x" ]; then
  if [ -f lib/functions.sh ]; then
    . lib/functions.sh
  elif [ -f ../lib/functions.sh ]; then
    cd ..
    . lib/functions.sh
  else
    echo 'Sorry, I need to be able to source functions.sh first.'
    echo Exiting...
    exit 1
  fi
fi

shout
shout '================================================================='
shout
shout 'Install PHP if not already installed, preferably PHP >= 7.2, but'
shout 'at least 7.0.'
shout 'If not PHP >= 7.2, build and install the "sodium" PHP extension.'
shout
shout '================================================================='
shout
pause

# Is PHP installed? If so, what version?
PHPVER='none'
PHPMAJOR=0
PHPPREFIX=''

# Find the php binary first. Let's hope 'which' is installed.
yum -y install which
PHPBIN="$( which php 2>/dev/null )"
if [ "x$PHPBIN" != "x" ]; then
  # This gives us 7.2.6 or similar
  PHPVER="$( $PHPBIN -i |
             grep '^PHP *Version' |
             head -1 |
             awk '{ print $NF }' )"
  shout Found PHP version "$PHPVER"
  # This gives us php72u
  PHPPREFIX="$( rpm -qf /usr/bin/php 2>/dev/null | sed -e 's/-cli-.*$//' | tail -1 )"
  # Reduce it to 72
  PHPVER="$( echo "$PHPVER" | sed -e 's/^\([^.]*\)\.\([^.]*\)\..*/\1\2/' )"
  # And 7
  PHPMAJOR="$( echo "$PHPVER" | cut -d. -f1 )"
  # Didn't work? Version=0 will force new installation
  if [ "x$PHPMAJOR" = "x" ]; then
    PHPMAJOR=0
    PHPVER=0
  fi
fi

# If on CentOS/RHEL 6 or 7, use IUS repo to get PHP 7
shout Good, you are on CentOS or RedHat 6 or later,
shout so PHP 7 is available ready-built.
if [ "$OSVER" -lt "8" ]; then
  shout Adding the IUS repo.
  #if [ "$OS" = "centos" -o "$OS" = "rocky" ]; then
  #  yum -y install https://centos"$OSVER".iuscommunity.org/ius-release.rpm
  #else
  #  yum -y install https://rhel"$OSVER".iuscommunity.org/ius-release.rpm
  #fi
  # IUS have merged the centos & rhel branches and moved to a CDN
  yum -y install https://repo.ius.io/ius-release-el"$OSVER".rpm
  #rpm --import /etc/pki/rpm-gpg/IUS-COMMUNITY-GPG-KEY
  # And the name of the GPG key file has changed too
  rpm --import /etc/pki/rpm-gpg/RPM-GPG-KEY-IUS-7
else
  shout Adding Remi\'s repo to get PHP with sodium support.
  shout For some strange reason it is missing from Rocky, CentOS and RedHat.
  yum -y install https://rpms.remirepo.net/enterprise/remi-release-${OSVER}.rpm
  rpm --import https://rpms.remirepo.net/RPM-GPG-KEY-remi
  Y="$( date '+%Y' )"
  for YEAR in $( seq 2017 "$Y" )
  do
    rpm --import https://rpms.remirepo.net/RPM-GPG-KEY-remi$YEAR
  done
fi


# These are the PHP modules that are the trouble-makers.
# Sometimes they are built in, sometimes they are RPMs, sometimes they have to be built.
HARDMODULES="imap sodium sqlite"

# If this is RedHat/CentOS 8 or later, we have modules to deal with.
# And 7.2 will always be available in that.
if [ "$OSVER" -ge "8" ]; then
  # Find latest PHP version available (this will give 8.1)
  # The -y automatically accepts all the GPG keys
  PHPVERDOTLATEST="$( yum -y --quiet module list --all php | grep '^php[[:space:]]' | awk '{ print $2 }' | grep '[78]\.' | sort -V | tail -1 )"
  shout 'Ignore warnings about it not affecting installed packages.'
  yum -y module reset php
  if [ "x$PHPVERDOTLATEST" != "x" ]; then
    yum -y module enable php:$PHPVERDOTLATEST
  else
    yum -y module enable php
  fi
  PHPPREFIXLATEST='php'
  # Reduce the version number to just digits
  PHPVERLATEST="$( echo $PHPVERDOTLATEST | tr -dc '0-9' )"
else
  # Find the most recent version of PHP we can get
  # It will give us a string like 'php72u' for 7.2.
  PHPPREFIXLATEST="php$( yum search all 'php-common' 2>/dev/null | egrep '^php[^-]+-common' | sed -e 's/^php//; s/-common.*$//' | grep '^[78]' | sort -n | tail -1 )"
  # This gives us '72'.
  PHPVERLATEST="$( echo "$PHPPREFIXLATEST" | tr -dc '0-9' )"
fi

# If there is no PHP installed, this bit is easy
if (( PHPVER == 0 )); then
  shout
  shout I am going to install the latest PHP I can find, which is "$PHPPREFIXLATEST" "$PHPVERLATEST".
  pause
  if (( OSVER >= 8 )); then
    # RHEL8 uses php-fpm not mod_php.
    yum -y install "$PHPPREFIXLATEST"-{cli,common,pdo,mysqlnd,ldap,intl,gd,enchant,json,mbstring,fpm}
    shout And enable the php-fpm service for Apache.
    systemctl enable php-fpm
  else
    yum -y install "$PHPPREFIXLATEST"-{cli,common,pdo,mysqlnd,ldap,intl,gd,enchant,json,mbstring} mod_"$PHPPREFIXLATEST"
  fi
  for F in $HARDMODULES; do
    shout "Don't worry if this fails..."
    yum -y install "$PHPPREFIXLATEST"-$F
  done
  PHPVER="$PHPVERLATEST"
  PHPPREFIX="$PHPPREFIXLATEST"
  pause
fi

# If there is PHP before 7.2 installed, and it can be upgraded,
# replace it with the latest
if (( PHPVER > 0 && PHPVER < 72 && PHPVER < PHPVERLATEST )); then
  shout
  shout "I need to replace your PHP installation (${PHPPREFIX}) with the latest"
  shout one I can find, which is "$PHPPREFIXLATEST" "$PHPVERLATEST".
  pause
  if (( OSVER >= 8 )); then
    shout "Switching module streams to PHP:$PHPVERDOTLATEST"
    yum -y module enable php:$PHPVERDOTLATEST
  else
    shout "I need the yum 'replace' plugin to do it first."
    yum -y install yum-plugin-replace
    shout Okay, this is the hard bit.
    shout I am going to replace all your PHP packages at once.
    shout 'This will produce lots of scary-looking warnings. Ignore them!'
    pause
    # Replacing php-common is enough to make yum replace everything.
    yum -y replace "$PHPPREFIX"-common --replace-with "$PHPPREFIXLATEST"-common
    shout Phew. Now to add any PHP modules you did not have installed.
    pause
  fi
    
  # We then just add any bits that are still missing.
  # Except sodium!
  yum -y install "$PHPPREFIXLATEST"-{cli,common,pdo,mysqlnd,ldap,intl,gd,enchant,json,mbstring} mod_"$PHPPREFIXLATEST"
  for F in $HARDMODULES; do
    shout "Don't worry if this fails..."
    yum -y install "$PHPPREFIXLATEST"-$F
  done
  pause
  PHPVER="$PHPVERLATEST"
  PHPPREFIX="$PHPPREFIXLATEST"
fi

## How do we now install sodium?
#if (( PHPVER >= 72 )); then
#  shout 'You are running PHP 7.2 or later, so the "sodium" extension is included'
#  shout "But don't worry if it fails to install..."
#  yum -y install "${PHPPREFIX}-sodium"
#fi

# Now we have hopefully got the php-sodium and php-sqlite PHP modules installed.
# But any of them might have failed.
# If any have, we need to install them with pecl.
DOWITHPEAR=""
for MODULE in $HARDMODULES; do
  if php -m | grep -qi $MODULE; then
    shout Found built-in PHP module $MODULE.
    continue
  fi
  if rpm --quiet -q "$PHPPREFIX"-"$MODULE"; then
    shout Found PHP package $MODULE installed.
    continue
  fi
  # php-imap is not available on RHEL8 (yet).
  if (( "$OSVER" >= 8 && "$MODULE" == "imap" )); then
    shout Skipping imap support on RedHat 8 as not available.
    continue
  fi
  shout Did not find PHP module $MODULE installed.
  DOWITHPEAR="$DOWITHPEAR $MODULE"
done


if [ "x$DOWITHPEAR" != "x" ]; then
  shout "We need to use PECL to install these extensions: $DOWITHPEAR"
  shout 'For that we will need some development tools.'
  pause
  if (( "$OSVER" >= 8 )); then
    yum -y module install php:$PHPVERDOTLATEST/devel
  elif (( "$OSVER" == 7 )); then
    yum -y --exclude='php*' groups install "Development Tools"
  else
    yum -y --exclude='php*' groupinstall "Development Tools"
  fi
  shout 'And the libraries and commands needed'
  yum -y install "$PHPPREFIX"-devel
  if (( OSVER >= 8 )); then
    yum -y install "$PHPPREFIX"-pear
  else
    shout "Don't worry if this one fails to install..."
    if yum -y install "$PHPPREFIX"-pear; then
      shout Yay, we found a recent pear.
    else
      shout "Don't worry, there is another one to try"
      yum -y install pear1u
    fi
  fi
  if (( OSVER >= 8 )); then
    :
    #shout 'And the development libraries'
    #for F in $( echo $DOWITHPEAR | sed -e 's/sodium/libsodium/' ); do
    #  yum -y install $F-devel
    #done
  else
    shout "And Remi's repo"
    yum -y install https://rpms.remirepo.net/enterprise/remi-release-${OSVER}.rpm
    rpm --import https://rpms.remirepo.net/RPM-GPG-KEY-remi
    Y="$( date '+%Y' )"
    for YEAR in $( seq 2017 "$Y" )
    do
      rpm --import https://rpms.remirepo.net/RPM-GPG-KEY-remi$YEAR
    done
    #shout 'And the development libraries'
    #for F in $( echo $DOWITHPEAR | sed -e 's/sodium/libsodium/' ); do
    #  yum -y install --enablerepo=remi $F-devel
    #done
  fi
  shout
  shout 'Now to build the' $DOWITHPEAR 'extension(s) using pecl'
  pause
  for F in $( echo $DOWITHPEAR | sed -e 's/sodium/libsodium/' ); do
    shout Building $F ...
    pecl install $F
    shout
    pause
  done
  for F in $DOWITHPEAR
  do
    shout "Enabling $F in PHP"
    cat > /etc/php.d/20-$F.ini <<EOINI
; ZendTo Enable $F extension module
extension=$F.so
EOINI
  done
fi

shout You should now have all the PHP packages installed.
exit 0
